@@ -22,11 +22,33 @@
  * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  * SOFTWARE. */
 
-#include "options.h"
+/* The format of the keyfiles is basically a raw dump of the buffer. Data types
+ * are specified in the transport draft - string is a 32-bit len then the
+ * non-null-terminated string, mp_int is a 32-bit len then the bignum data.
+ * The actual functions are buf_put_rsa_priv_key() and buf_put_dss_priv_key()
+
+ * RSA:
+ * string	"ssh-rsa"
+ * mp_int	e
+ * mp_int	n
+ * mp_int	d
+ * mp_int	p (newer versions only)
+ * mp_int	q (newer versions only) 
+ *
+ * DSS:
+ * string	"ssh-dss"
+ * mp_int	p
+ * mp_int	q
+ * mp_int	g
+ * mp_int	y
+ * mp_int	x
+ *
+ */
+#include "includes.h"
 #include "runopts.h"
 #include "signkey.h"
 #include "buffer.h"
-#include "util.h"
+#include "dbutil.h"
 
 #include "genrsa.h"
 #include "gendss.h"
@@ -39,6 +61,8 @@ static void printhelp(char * progname);
 #define DSS_SIZE (1024/8) /* 1024 bit */
 
 static void buf_writefile(buffer * buf, const char * filename);
+
+/* Print a help message */
 static void printhelp(char * progname) {
 
 	fprintf(stderr, "Usage: %s -t <type> -f <filename> [-s bits]\n"
@@ -113,6 +137,7 @@ int main(int argc, char ** argv) {
 #ifdef DROPBEAR_DSS
 				"dss\n"
 #endif
+				"\n"
 			   );
 		printhelp(argv[0]);
 		exit(EXIT_FAILURE);
@@ -170,6 +195,9 @@ int main(int argc, char ** argv) {
 	fprintf(stderr, "Will output %d bit %s secret key to '%s'\n", keysize*8,
 			typetext, filename);
 
+	/* don't want the file readable by others */
+	umask(077);
+
 	/* now we can generate the key */
 	key = new_sign_key();
 	
@@ -205,6 +233,7 @@ int main(int argc, char ** argv) {
 	return EXIT_SUCCESS;
 }
 
+/* Write a buffer to a file specified, failing if the file exists */
 static void buf_writefile(buffer * buf, const char * filename) {
 
 	int fd;
