@@ -22,12 +22,8 @@
  * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  * SOFTWARE. */
 
-#include <sys/stat.h>
-#include <fcntl.h>
-#include <unistd.h>
-
-#include "options.h"
-#include "util.h"
+#include "includes.h"
+#include "dbutil.h"
 #include "algo.h"
 #include "buffer.h"
 #include "session.h"
@@ -36,7 +32,6 @@
 #include "packet.h"
 #include "bignum.h"
 #include "random.h"
-#include "libtomcrypt/mycrypt.h"
 
 /* diffie-hellman-group1-sha1 values for g and p */
 const unsigned char dh_p_val[] = {
@@ -51,19 +46,24 @@ const unsigned char dh_p_val[] = {
 	0xEE, 0x38, 0x6B, 0xFB, 0x5A, 0x89, 0x9F, 0xA5, 0xAE, 0x9F, 0x24, 0x11,
 	0x7C, 0x4B, 0x1F, 0xE6, 0x49, 0x28, 0x66, 0x51, 0xEC, 0xE6, 0x53, 0x81,
 	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF};
-
 #define DH_P_LEN 128
-const unsigned int dh_p_digits = 310;
 
 const unsigned int dh_g_val = 2;
 
 static void read_kex();
 static void gen_new_keys();
+static void gen_new_zstreams();
 /* helper function for gen_new_keys */
 static void hashkeys(unsigned char *out, int outlen, 
 		const hash_state * hs, unsigned const char X);
 static void send_msg_kexdh_reply(mp_int *dh_e);
 
+/* Executed upon receiving a kexinit message from the client to initiate
+ * key exchange. If we haven't already done so, we send the list of our
+ * preferred algorithms. The client's requested algorithms are processed,
+ * and we calculate the first portion of the key-exchange-hash for used
+ * later in the key exchange. No response is sent, as the client should
+ * initiate the diffie-hellman key exchange */
 void recv_msg_kexinit() {
 	
 	TRACE(("enter recv_msg_kexinit"));
@@ -105,6 +105,7 @@ void recv_msg_kexinit() {
 	TRACE(("leave recv_msg_kexinit"));
 }
 
+/* Bring new keys into use after a key exchange, and let the client know*/
 void send_msg_newkeys() {
 
 	TRACE(("enter send_msg_newkeys"));
@@ -126,6 +127,7 @@ void send_msg_newkeys() {
 	TRACE(("leave send_msg_newkeys"));
 }
 
+/* Bring the new keys into use after a key exchange */
 void recv_msg_newkeys() {
 
 	TRACE(("enter recv_msg_newkeys"));
@@ -143,7 +145,7 @@ void recv_msg_newkeys() {
 	TRACE(("leave recv_msg_newkeys"));
 }
 
-/* set the kex state variables to initial values, ready to receive a new
+/* Set the kex state variables to initial values, ready to receive a new
  * SSH_MSG_KEXINIT (or send one) */
 void kexinitialise() {
 
@@ -171,12 +173,14 @@ void kexinitialise() {
 
 }
 
-/* helper function for gen_new_keys, creates a hash. It makes a copy of the
+/* Helper function for gen_new_keys, creates a hash. It makes a copy of the
  * already initialised hash_state hs, which should already have processed
  * the dh_K and hash, since these are common. X is the letter 'A', 'B' etc.
  * out must have at least min(SHA1_HASH_SIZE, outlen) bytes allocated.
  * The output will only be expanded once, since that is all that is required
- * (for 3DES and SHA, with 24 and 20 bytes respectively). */
+ * (for 3DES and SHA, with 24 and 20 bytes respectively). 
+ *
+ * See Section 5.2 of the IETF secsh Transport Draft for details */
 static void hashkeys(unsigned char *out, int outlen, 
 		const hash_state * hs, const unsigned char X) {
 
@@ -196,8 +200,12 @@ static void hashkeys(unsigned char *out, int outlen,
 	}
 }
 
-/* generate the actual encryption/integrity keys, using the results of the
- * key exchange */
+/* Generate the actual encryption/integrity keys, using the results of the
+ * key exchange, as specified in section 5.2 of the IETF secsh-transport
+ * draft. This occurs after the DH key-exchange.
+ *
+ * ses.newkeys is the new set of keys which are generated, these are only
+ * taken into use after both sides have sent a newkeys message */
 static void gen_new_keys() {
 
 	unsigned char IV[MAX_IV_LEN];
@@ -247,15 +255,27 @@ static void gen_new_keys() {
 	keysize = ses.newkeys->trans_algo_mac->keysize;
 	hashkeys(ses.newkeys->transmackey, keysize, &hs, 'F');
 
-	/* compression */
 #ifndef DISABLE_ZLIB
-	/* create new zstreams */
-	TRACE(("enter ZLIB section of genkeys"));
+	gen_new_zstreams();
+#endif
+	
+	/* Switch over to the new keys */
+	m_burn(ses.keys, sizeof(struct key_context));
+	m_free(ses.keys);
+	ses.keys = ses.newkeys;
+	ses.newkeys = NULL;
 
+	TRACE(("leave gen_new_keys"));
+}
+
+#ifndef DISABLE_ZLIB
+/* Set up new zlib compression streams, close the old ones. Only
+ * called from gen_new_keys() */
+static void gen_new_zstreams() {
+
+	/* create new zstreams */
 	if (ses.newkeys->recv_algo_comp == DROPBEAR_COMP_ZLIB) {
 		ses.newkeys->recv_zstream = (z_streamp)m_malloc(sizeof(z_stream));
-		ses.newkeys->recv_zstream->next_in = Z_NULL;
-		ses.newkeys->recv_zstream->avail_in = 0;
 		ses.newkeys->recv_zstream->zalloc = Z_NULL;
 		ses.newkeys->recv_zstream->zfree = Z_NULL;
 		
@@ -265,12 +285,9 @@ static void gen_new_keys() {
 	} else {
 		ses.newkeys->recv_zstream = NULL;
 	}
-	TRACE(("past inflateInit"));
 
 	if (ses.newkeys->trans_algo_comp == DROPBEAR_COMP_ZLIB) {
 		ses.newkeys->trans_zstream = (z_streamp)m_malloc(sizeof(z_stream));
-		ses.newkeys->trans_zstream->next_in = Z_NULL;
-		ses.newkeys->trans_zstream->avail_in = 0;
 		ses.newkeys->trans_zstream->zalloc = Z_NULL;
 		ses.newkeys->trans_zstream->zfree = Z_NULL;
 	
@@ -281,41 +298,29 @@ static void gen_new_keys() {
 	} else {
 		ses.newkeys->trans_zstream = NULL;
 	}
-	TRACE(("past deflateInit"));
-
 	
 	/* clean up old keys */
 	if (ses.keys->recv_zstream != NULL) {
-		int ret;
-		ret = inflateEnd(ses.keys->recv_zstream);
-		if (ret == Z_STREAM_ERROR) {
+		if (inflateEnd(ses.keys->recv_zstream) == Z_STREAM_ERROR) {
 			/* Z_DATA_ERROR is ok, just means that stream isn't ended */
 			dropbear_exit("crypto error");
 		}
 		m_free(ses.keys->recv_zstream);
 	}
 	if (ses.keys->trans_zstream != NULL) {
-		int ret;
-		ret = deflateEnd(ses.keys->trans_zstream);
-		/* Z_DATA_ERROR is ok, just means that stream isn't ended */
-		if (ret == Z_STREAM_ERROR) {
+		if (deflateEnd(ses.keys->trans_zstream) == Z_STREAM_ERROR) {
+			/* Z_DATA_ERROR is ok, just means that stream isn't ended */
 			dropbear_exit("crypto error");
 		}
 		m_free(ses.keys->trans_zstream);
 	}
-	TRACE(("leave ZLIB section of genkeys"));
+}
 #endif
 
-	
-	/* change the keys */
-	m_burn((void*)ses.keys, sizeof(struct key_context));
-	m_free(ses.keys);
-	ses.keys = ses.newkeys;
-	ses.newkeys = NULL;
-
-	TRACE(("leave gen_new_keys"));
-}
-
+/* Handle a diffie-hellman key exchange initialisation. This involves
+ * calculating a session key reply value, and corresponding hash. These
+ * are carried out by send_msg_kexdh_reply(). recv_msg_kexdh_init() calls
+ * that function, then brings the new keys into use */
 void recv_msg_kexdh_init() {
 
 	mp_int dh_e;
@@ -337,7 +342,12 @@ void recv_msg_kexdh_init() {
 	TRACE(("leave recv_msg_kexdh_init"));
 }
 	
-/* some mp_ints are cleared before the end to attempt to minimise memory */
+/* Generate our side of the diffie-hellman key exchange value (dh_f), and
+ * calculate the session key using the diffie-hellman algorithm. Following
+ * that, the session hash is calculated, and signed with RSA or DSS. The
+ * result is sent to the client. 
+ *
+ * See the ietf-secsh-transport draft, section 6, for details */
 static void send_msg_kexdh_reply(mp_int *dh_e) {
 
 	mp_int dh_p, dh_q, dh_g, dh_y, dh_f;
@@ -349,12 +359,7 @@ static void send_msg_kexdh_reply(mp_int *dh_e) {
 	
 	assert(ses.kexstate.recvkexinit);
 
-	m_mp_init(&dh_g);
-	/*mp_set_prec(dh_p_digits);*/ /* we may as well set the size right to start */
-	m_mp_init(&dh_p);
-	m_mp_init(&dh_q);
-	m_mp_init(&dh_y);
-	m_mp_init(&dh_f);
+	m_mp_init_multi(&dh_g, &dh_p, &dh_q, &dh_y, &dh_f, NULL);
 
 	/* read the prime and generator*/
 	if (mp_read_unsigned_bin(&dh_p, (unsigned char*)dh_p_val, DH_P_LEN)
@@ -378,12 +383,12 @@ static void send_msg_kexdh_reply(mp_int *dh_e) {
 
 	/* calculate our random value dh_y */
 	do {
-		assert(dh_q_len <= sizeof(randbuf));
+		assert((unsigned int)dh_q_len <= sizeof(randbuf));
 		genrandom(randbuf, dh_q_len);
 		if (mp_read_unsigned_bin(&dh_y, randbuf, dh_q_len) != MP_OKAY) {
 			dropbear_exit("Diffie-Hellman error");
 		}
-	} while (mp_cmp(&dh_y, &dh_q) >= 0 || mp_cmp_d(&dh_y, 0) <= 0);
+	} while (mp_cmp(&dh_y, &dh_q) == MP_GT || mp_cmp_d(&dh_y, 0) != MP_GT);
 
 	/* f = g^y mod p */
 	if (mp_exptmod(&dh_g, &dh_y, &dh_p, &dh_f) != MP_OKAY) {
@@ -399,9 +404,7 @@ static void send_msg_kexdh_reply(mp_int *dh_e) {
 	}
 
 	/* clear no longer needed vars */
-	mp_clear(&dh_y);
-	mp_clear(&dh_p);
-	mp_clear(&dh_q);
+	mp_clear_multi(&dh_y, &dh_p, &dh_q, NULL);
 
 	/* Create the remainder of the hash buffer, to generate the exchange hash */
 	/* K_S, the host key */
@@ -414,33 +417,17 @@ static void send_msg_kexdh_reply(mp_int *dh_e) {
 	/* K, the shared secret */
 	buf_putmpint(ses.kexhashbuf, ses.dh_K);
 
-	/* calculate the hash H */
+	/* calculate the hash H to sign */
 	sha1_init(&hs);
 	buf_setpos(ses.kexhashbuf, 0);
 	sha1_process(&hs, buf_getptr(ses.kexhashbuf, ses.kexhashbuf->len),
 			ses.kexhashbuf->len);
 	sha1_done(&hs, ses.hash);
-#if defined(DEBUG_KEXHASH) && defined(DEBUG_TRACE)
-	{
-		int hashfd;
-		int hashwritelen;
-		hashfd = creat("/tmp/hashbufm", S_IRUSR|S_IWUSR|S_IROTH|S_IRGRP);
-	
-		hashwritelen = write(hashfd, 
-				buf_getptr(ses.kexhashbuf, ses.kexhashbuf->len),
-				ses.kexhashbuf->len);
-		if (hashwritelen != ses.kexhashbuf->len) {
-			dropbear_exit("bad hashwritelen debugging");
-		}
-		close(hashfd);
-		printhex(ses.hash, SHA1_HASH_SIZE);
-	}
-#endif /* DEBUG_KEXHASH */
 	buf_free(ses.kexhashbuf);
 	ses.kexhashbuf = NULL;
 	
 	/* first time around, we set the session_id to H */
-	if (!ses.session_id) {
+	if (ses.session_id == NULL) {
 		/* create the session_id, this never needs freeing */
 		ses.session_id = (unsigned char*)m_malloc(SHA1_HASH_SIZE);
 		memcpy(ses.session_id, ses.hash, SHA1_HASH_SIZE);
@@ -556,7 +543,7 @@ static void read_kex() {
 	}
 
 	/* reserved for future extensions */
-	/* buf_getint(ses.payload);*/
+	buf_getint(ses.payload);
 
 	return;
 
@@ -565,6 +552,7 @@ error:
 
 }
 
+/* Send our list of algorithms we can use */
 void send_msg_kexinit() {
 
 	CHECKCLEARTOWRITE();
