@@ -30,7 +30,7 @@ static const struct {
 },
 #endif
 #ifdef ECC192
-{  
+{
     24,
    "ECC-192",
    /* prime */
@@ -65,7 +65,7 @@ static const struct {
 
    /* Gx */
    "jpqOf1BHus6Yd/pyhyVpP",
- 
+
    /* Gy */
    "3FCtyo2yHA5SFjkCGbYxbOvNeChwS+j6wSIwck",
 },
@@ -88,7 +88,7 @@ static const struct {
 
    /* Gy */
    "4/ZGkB+6d+RZkVhIdmFdXOhpZDNQp5UpiksG6Wtlr7r"
-}, 
+},
 #endif
 #ifdef ECC384
 {
@@ -97,11 +97,11 @@ static const struct {
    /* prime */
    "//////////////////////////////////////////x/////00000000003/"
    "////",
-   
+
    /* B */
    "ip4lf+8+v+IOZWLhu/Wj6HWTd6x+WK4I0nG8Zr0JXrh6LZcDYYxHdIg5oEtJ"
    "x2hl",
-   
+
    /* Order */
    "////////////////////////////////nsDDWVGtBTzO6WsoIB2dUkpi6MhC"
    "nIbp",
@@ -109,7 +109,7 @@ static const struct {
    /* Gx and Gy */
    "geVA8hwB1JUEiSSUyo2jT6uTEsABfvkOMVT1u89KAZXL0l9TlrKfR3fKNZXo"
    "TWgt",
-   
+
    "DXVUIfOcB6zTdfY/afBSAVZq7RqecXHywTen4xNmkC0AOB7E7Nw1dNf37NoG"
    "wWvV"
 },
@@ -118,22 +118,22 @@ static const struct {
 {
    65,
    "ECC-521",
-   /* prime */ 
+   /* prime */
    "V///////////////////////////////////////////////////////////"
    "///////////////////////////",
- 
+
    /* B */
    "56LFhbXZXoQ7vAQ8Q2sXK3kejfoMvcp5VEuj8cHZl49uLOPEL7iVfDx5bB0l"
    "JknlmSrSz+8FImqyUz57zHhK3y0",
- 
-   /* Order */ 
+
+   /* Order */
    "V//////////////////////////////////////////+b66XuE/BvPhVym1I"
    "FS9fT0xjScuYPn7hhjljnwHE6G9",
 
    /* Gx and Gy */
    "CQ5ZWQt10JfpPu+osOZbRH2d6I1EGK/jI7uAAzWQqqzkg5BNdVlvrae/Xt19"
    "wB/gDupIBF1XMf2c/b+VZ72vRrc",
-   
+
    "HWvAMfucZl015oANxGiVHlPcFL4ILURH6WNhxqN9pvcB9VkSfbUz2P0nL2v0"
    "J+j1s4rF726edB2G8Y+b7QVqMPG",
 },
@@ -189,7 +189,7 @@ void ecc_find_base(void)
       mp_sqrmod(&tmp2, &p, &tmp2);
 
       /* tmp2 should equal tmp1 */
-   } while (mp_cmp(&tmp1, &tmp2)); 
+   } while (mp_cmp(&tmp1, &tmp2));
 
    /* now output values in way that libtomcrypt wants */
    mp_todecimal(&p, buf);
@@ -235,8 +235,13 @@ static ecc_point *new_point(void)
 
 static void del_point(ecc_point *p)
 {
-   mp_clear_multi(&p->x, &p->y, NULL);
-   XFREE(p);
+   /* prevents free'ing null arguments */
+   if (p == NULL) {
+      return;
+   } else {
+      mp_clear_multi(&p->x, &p->y, NULL);
+      XFREE(p);
+   }
 }
 
 /* double a point R = 2P, R can be P*/
@@ -244,8 +249,8 @@ static int dbl_point(ecc_point *P, ecc_point *R, mp_int *modulus, mp_int *mu)
 {
    mp_int s, tmp, tmpx;
    int res;
-   
-   if (mp_init_multi(&s, &tmp, &tmpx, NULL) != MP_OKAY) { 
+
+   if (mp_init_multi(&s, &tmp, &tmpx, NULL) != MP_OKAY) {
       return CRYPT_MEM;
    }
 
@@ -288,8 +293,8 @@ static int add_point(ecc_point *P, ecc_point *Q, ecc_point *R, mp_int *modulus,
 {
    mp_int s, tmp, tmpx;
    int res;
-   
-   if (mp_init(&tmp) != MP_OKAY) { 
+
+   if (mp_init(&tmp) != MP_OKAY) {
       return CRYPT_MEM;
    }
 
@@ -298,14 +303,14 @@ static int add_point(ecc_point *P, ecc_point *Q, ecc_point *R, mp_int *modulus,
       mp_clear(&tmp);
       return CRYPT_MEM;
    }
-   
+
    if (mp_cmp(&P->x, &Q->x) == MP_EQ)
       if (mp_cmp(&P->y, &Q->y) == MP_EQ || mp_cmp(&P->y, &tmp) == MP_EQ) {
          mp_clear(&tmp);
          return dbl_point(P, R, modulus, mu);
       }
 
-   if (mp_init_multi(&tmpx, &s, NULL) != MP_OKAY) { 
+   if (mp_init_multi(&tmpx, &s, NULL) != MP_OKAY) {
       mp_clear(&tmp);
       return CRYPT_MEM;
    }
@@ -344,174 +349,176 @@ done:
    return res;
 }
 
+/* size of sliding window, don't change this! */
+#define WINSIZE 4
+
 /* perform R = kG where k == integer and G == ecc_point */
 static int ecc_mulmod(mp_int *k, ecc_point *G, ecc_point *R, mp_int *modulus)
 {
-   ecc_point *tG, *M[30];
-   int i, j, z, res, Q;
-   mp_digit d;
-   unsigned char bits[150], m, first;
+   ecc_point *tG, *M[8];
+   int i, j, res;
    mp_int mu;
-   
-   
-   if ((USED(k) * MP_DIGIT_BIT) > 256) {
-      Q = 5;
-   } else {
-      Q = 4;
-   }
-   
-   if (mp_init(&mu) != MP_OKAY) {
-      return CRYPT_MEM;
-   }
-   
+   mp_digit buf;
+   int     first, bitbuf, bitcpy, bitcnt, mode, digidx;
+
   /* init barrett reduction */
-  mp_set(&mu, 1); 
-  mp_lshd(&mu, 2 * USED(modulus));
-  if (mp_div(&mu, modulus, &mu, NULL) != MP_OKAY) {
-    mp_clear(&mu);
-    return CRYPT_MEM;
+  if (mp_init(&mu) != MP_OKAY) {
+      return CRYPT_MEM;
+  }
+  if (mp_reduce_setup(&mu, modulus) != MP_OKAY) {
+      mp_clear(&mu);
+      return CRYPT_MEM;
+  }
+
+  /* alloc ram for window temps */
+  for (i = 0; i < 8; i++) {
+      M[i] = new_point();
+      if (M[i] == NULL) {
+         for (j = 0; j < i; j++) {
+             del_point(M[j]);
+         }
+         mp_clear(&mu);
+         return CRYPT_MEM;
+      }
   }
-   
-   
-   /* init M tab (alloc here, calculate below)
-    
-    This table holds the first 2^Q multiples of the input base point G, that is 
-    
-       M[x] = x * G
-       
-    Where G is the point and x is a scalar.  The implementation is optimized
-    since M[0] == 0 and M[1] == G so there is no need to waste space for those.  In
-    effect M'[x] == M[x+2] where M'[] is the table we make.  If M[0] or M[1] are needed
-    we handle them with if statements.   
-   
-   */
-   for (i = 0; i < ((1<<Q)-2); i++) {
-       M[i] = new_point();
-       if (M[i] == NULL) {
-          for (j = 0; j < i; j++) {
-              del_point(M[j]);
-          }
-          mp_clear(&mu);
-          return CRYPT_MEM;
-       }
-   }
-   
-   /* get bits of k in groupings of Q 
-   
-    The multiplicand is read in groupings of four bits.  This is because the multiplication
-    routine is a Q-ary left-to-write (see HAC chapter 14, algorithm 14.82).
-   */
-   first = m = (unsigned char)0;
-   for (z = i = 0; z < (int)USED(k); z++) {
-       /* grab a digit from the mp_int, these have MP_DIGIT_BIT bits in them */
-       d = DIGIT(k, z);
-       for (j = 0; j < (int)MP_DIGIT_BIT; j++) {
-           /* OR the bits against an accumulator */
-           first |= (d&1)<<(unsigned)(m++);
-           /* if the bit count is Q then we have a Q-bit word ready */
-           if (m == (unsigned char)Q) {
-              /* store the four bit word and reset counters */
-              bits[i++] = first;
-              first = m = (unsigned char)0;
-           }
-           /* shift the digit down to extract the next bit */
-           d >>= 1;
-       }
-   }
-   
-   /* residue of multiplicand [if any] */
-   if (m) {
-      bits[i++] = first;
-   }
 
    /* make a copy of G incase R==G */
    tG = new_point();
-   if (tG == NULL)                                               { goto error; }
+   if (tG == NULL)                                                    { goto error; }
 
-   /* skip leading digits which are zero */   
-   --i; while (i != 0 && bits[i] == (unsigned char)0) { --i; }
-   
-   /* if the multiplicand has no non-zero 4-bit words its invalid. */
-   if (i == 0) {
-      res = CRYPT_INVALID_ARG;
-      goto done;
-   }
-   
-   /* now calc the M tab, note that there are only 2^Q - 2 spots, the normal M[0] is a no-op, and M[1] is the input
-      point (saves ram)
-   */
-   
-   /* M[0] now is 2*G */
+   /* calc the M tab, which holds kG for k==8..15 */
+   /* M[0] == 8G */
    if (dbl_point(G, M[0], modulus, &mu) != CRYPT_OK)                  { goto error; }
-   for (j = 1; j < ((1<<Q)-2); j++) {
-       if (add_point(M[j-1], G, M[j], modulus, &mu) != CRYPT_OK)      { goto error; }
-   }
-  
-   /* tG = G */
-   if (mp_copy(&G->x, &tG->x) != MP_OKAY)                        { goto error; }
-   if (mp_copy(&G->y, &tG->y) != MP_OKAY)                        { goto error; }
+   if (dbl_point(M[0], M[0], modulus, &mu) != CRYPT_OK)               { goto error; }
+   if (dbl_point(M[0], M[0], modulus, &mu) != CRYPT_OK)               { goto error; }
 
-   /* set result M[bits[i]] */
-   if (bits[i] == (unsigned char)1) {
-     if (mp_copy(&G->x, &R->x) != MP_OKAY)                       { goto error; }
-     if (mp_copy(&G->y, &R->y) != MP_OKAY)                       { goto error; }
-   } else if (bits[i] >= (unsigned char)2) {
-     if (mp_copy(&M[(int)bits[i]-2]->x, &R->x) != MP_OKAY)       { goto error; }
-     if (mp_copy(&M[(int)bits[i]-2]->y, &R->y) != MP_OKAY)       { goto error; }
+   /* now find (8+k)G for k=1..7 */
+   for (j = 9; j < 16; j++) {
+       if (add_point(M[j-9], G, M[j-8], modulus, &mu) != CRYPT_OK)    { goto error; }
    }
-   
-   while (--i >= 0) {
-       /* double */
-       for (j = 0; j < Q; j++) {
-          if (dbl_point(R, R, modulus, &mu) != CRYPT_OK)              { goto error; }
+
+   /* tG = G */
+   if (mp_copy(&G->x, &tG->x) != MP_OKAY)                             { goto error; }
+   if (mp_copy(&G->y, &tG->y) != MP_OKAY)                             { goto error; }
+
+   /* setup sliding window */
+   mode   = 0;
+   bitcnt = 1;
+   buf    = 0;
+   digidx = k->used - 1;
+   bitcpy = bitbuf = 0;
+   first  = 1;
+
+   /* perform ops */
+   for (;;) {
+     /* grab next digit as required */
+     if (--bitcnt == 0) {
+       if (digidx == -1) {
+          break;
        }
-       
-       /* now based on the value of bits[i] we do ops */
-       if (bits[i] == (unsigned char)0) {
-          /* nop */
-       } else if (bits[i] == (unsigned char)1) {
-          /* add base point */
-          if (add_point(R, tG, R, modulus, &mu) != CRYPT_OK)          { goto error; }
+       buf = k->dp[digidx--];
+       bitcnt = (int) DIGIT_BIT;
+     }
+
+     /* grab the next msb from the multiplicand */
+     i = (buf >> (DIGIT_BIT - 1)) & 1;
+     buf <<= 1;
+
+     /* skip leading zero bits */
+     if (mode == 0 && i == 0) {
+        continue;
+     }
+
+     /* if the bit is zero and mode == 1 then we double */
+     if (mode == 1 && i == 0) {
+        if (dbl_point(R, R, modulus, &mu) != CRYPT_OK)                          { goto error; }
+        continue;
+     }
+
+     /* else we add it to the window */
+     bitbuf |= (i << (WINSIZE - ++bitcpy));
+     mode = 2;
+
+     if (bitcpy == WINSIZE) {
+       /* if this is the first window we do a simple copy */
+       if (first == 1) {
+          /* R = kG [k = first window] */
+          if (mp_copy(&M[bitbuf-8]->x, &R->x) != MP_OKAY)                       { goto error; }
+          if (mp_copy(&M[bitbuf-8]->y, &R->y) != MP_OKAY)                       { goto error; }
+          first = 0;
        } else {
-          /* other case */
-          if (add_point(R, M[(int)bits[i] - 2], R, modulus, &mu) != CRYPT_OK) { goto error; }
+         /* normal window */
+         /* ok window is filled so double as required and add  */
+         /* double first */
+         for (j = 0; j < WINSIZE; j++) {
+           if (dbl_point(R, R, modulus, &mu) != CRYPT_OK)                       { goto error; }
+         }
+
+         /* then add, bitbuf will be 8..15 [8..2^WINSIZE] guaranteed */
+         if (add_point(R, M[bitbuf-8], R, modulus, &mu) != CRYPT_OK)            { goto error; }
        }
+       /* empty window and reset */
+       bitcpy = bitbuf = 0;
+       mode = 1;
+    }
+  }
+
+   /* if bits remain then double/add */
+   if (mode == 2 && bitcpy > 0) {
+     /* double then add */
+     for (j = 0; j < bitcpy; j++) {
+       /* only double if we have had at least one add first */
+       if (first == 0) {
+          if (dbl_point(R, R, modulus, &mu) != CRYPT_OK)                       { goto error; }
+       }
+
+       bitbuf <<= 1;
+       if ((bitbuf & (1 << WINSIZE)) != 0) {
+         if (first == 1){
+            /* first add, so copy */
+            if (mp_copy(&tG->x, &R->x) != MP_OKAY)                             { goto error; }
+            if (mp_copy(&tG->y, &R->y) != MP_OKAY)                             { goto error; }
+            first = 0;
+         } else {
+            /* then add */
+            if (add_point(R, tG, R, modulus, &mu) != CRYPT_OK)                 { goto error; }
+         }
+       }
+     }
    }
-   
-   res = CRYPT_OK; 
+   res = CRYPT_OK;
    goto done;
 error:
    res = CRYPT_MEM;
 done:
    del_point(tG);
-   for (i = 0; i < ((1<<Q)-2); i++) {
+   for (i = 0; i < 8; i++) {
        del_point(M[i]);
    }
    mp_clear(&mu);
-#ifdef CLEAN_STACK
-   zeromem(bits, sizeof(bits)); 
-#endif
    return res;
 }
 
+#undef WINSIZE
+
 int ecc_test(void)
 {
    mp_int     modulus, order;
    ecc_point  *G, *GG;
    int i, res, primality;
 
-   if (mp_init_multi(&modulus, &order, NULL) != MP_OKAY) { 
+   if (mp_init_multi(&modulus, &order, NULL) != MP_OKAY) {
       return CRYPT_MEM;
    }
 
    G   = new_point();
-   if (G == NULL) { 
+   if (G == NULL) {
       mp_clear_multi(&modulus, &order, NULL);
       return CRYPT_MEM;
    }
 
    GG  = new_point();
-   if (GG == NULL) { 
+   if (GG == NULL) {
       mp_clear_multi(&modulus, &order, NULL);
       del_point(G);
       return CRYPT_MEM;
@@ -521,8 +528,8 @@ int ecc_test(void)
        #if 0
           printf("Testing %d\n", sets[i].size);
        #endif
-       if (mp_read_radix(&modulus, (unsigned char *)sets[i].prime, 64) != MP_OKAY)   { goto error; }
-       if (mp_read_radix(&order, (unsigned char *)sets[i].order, 64) != MP_OKAY)     { goto error; }
+       if (mp_read_radix(&modulus, (char *)sets[i].prime, 64) != MP_OKAY)   { goto error; }
+       if (mp_read_radix(&order, (char *)sets[i].order, 64) != MP_OKAY)     { goto error; }
 
        /* is prime actually prime? */
        if (is_prime(&modulus, &primality) != CRYPT_OK)           { goto error; }
@@ -530,7 +537,7 @@ int ecc_test(void)
           res = CRYPT_FAIL_TESTVECTOR;
           goto done1;
        }
-  
+
        /* is order prime ? */
        if (is_prime(&order, &primality) != CRYPT_OK)             { goto error; }
        if (primality == 0) {
@@ -538,8 +545,8 @@ int ecc_test(void)
           goto done1;
        }
 
-       if (mp_read_radix(&G->x, (unsigned char *)sets[i].Gx, 64) != MP_OKAY) { goto error; }
-       if (mp_read_radix(&G->y, (unsigned char *)sets[i].Gy, 64) != MP_OKAY) { goto error; }
+       if (mp_read_radix(&G->x, (char *)sets[i].Gx, 64) != MP_OKAY) { goto error; }
+       if (mp_read_radix(&G->y, (char *)sets[i].Gy, 64) != MP_OKAY) { goto error; }
 
        /* then we should have G == (order + 1)G */
        if (mp_add_d(&order, 1, &order) != MP_OKAY)                  { goto error; }
@@ -569,11 +576,11 @@ void ecc_sizes(int *low, int *high)
  *low = INT_MAX;
  *high = 0;
  for (i = 0; sets[i].size != 0; i++) {
-     if (sets[i].size < *low)  { 
-        *low  = sets[i].size; 
+     if (sets[i].size < *low)  {
+        *low  = sets[i].size;
      }
-     if (sets[i].size > *high) { 
-        *high = sets[i].size; 
+     if (sets[i].size > *high) {
+        *high = sets[i].size;
      }
  }
 }
@@ -583,7 +590,7 @@ int ecc_make_key(prng_state *prng, int wprng, int keysize, ecc_key *key)
    int x, res, err;
    ecc_point *base;
    mp_int prime;
-   unsigned char buf[4096];
+   unsigned char buf[128];
 
    _ARGCHK(key != NULL);
 
@@ -596,7 +603,7 @@ int ecc_make_key(prng_state *prng, int wprng, int keysize, ecc_key *key)
    for (x = 0; (keysize > sets[x].size) && (sets[x].size != 0); x++);
    keysize = sets[x].size;
 
-   if (sets[x].size == 0) { 
+   if (sets[x].size == 0) {
       return CRYPT_INVALID_KEYSIZE;
    }
    key->idx = x;
@@ -607,7 +614,7 @@ int ecc_make_key(prng_state *prng, int wprng, int keysize, ecc_key *key)
    }
 
    /* setup the key variables */
-   if (mp_init_multi(&key->pubkey.x, &key->pubkey.y, &key->k, &prime, NULL) != MP_OKAY) { 
+   if (mp_init_multi(&key->pubkey.x, &key->pubkey.y, &key->k, &prime, NULL) != MP_OKAY) {
       return CRYPT_MEM;
    }
    base = new_point();
@@ -617,15 +624,15 @@ int ecc_make_key(prng_state *prng, int wprng, int keysize, ecc_key *key)
    }
 
    /* read in the specs for this key */
-   if (mp_read_radix(&prime, (unsigned char *)sets[key->idx].prime, 64) != MP_OKAY)  { goto error; }
-   if (mp_read_radix(&base->x, (unsigned char *)sets[key->idx].Gx, 64) != MP_OKAY)   { goto error; }
-   if (mp_read_radix(&base->y, (unsigned char *)sets[key->idx].Gy, 64) != MP_OKAY)   { goto error; }
+   if (mp_read_radix(&prime, (char *)sets[key->idx].prime, 64) != MP_OKAY)  { goto error; }
+   if (mp_read_radix(&base->x, (char *)sets[key->idx].Gx, 64) != MP_OKAY)   { goto error; }
+   if (mp_read_radix(&base->y, (char *)sets[key->idx].Gy, 64) != MP_OKAY)   { goto error; }
    if (mp_read_unsigned_bin(&key->k, (unsigned char *)buf, keysize) != MP_OKAY)      { goto error; }
 
    /* make the public key */
    if (ecc_mulmod(&key->k, base, &key->pubkey, &prime) != CRYPT_OK) { goto error; }
    key->type = PK_PRIVATE;
-   
+
    /* shrink key */
    if (mp_shrink(&key->k) != MP_OKAY)          { goto error; }
    if (mp_shrink(&key->pubkey.x) != MP_OKAY)   { goto error; }
@@ -664,12 +671,12 @@ static int compress_y_point(ecc_point *pt, int idx, int *result)
    }
 
    /* get x^3 - 3x + b */
-   if (mp_read_radix(&p, (unsigned char *)sets[idx].B, 64) != MP_OKAY) { goto error; } /* p = B */
+   if (mp_read_radix(&p, (char *)sets[idx].B, 64) != MP_OKAY) { goto error; } /* p = B */
    if (mp_expt_d(&pt->x, 3, &tmp) != MP_OKAY)              { goto error; } /* tmp = pX^3  */
    if (mp_mul_d(&pt->x, 3, &tmp2) != MP_OKAY)              { goto error; } /* tmp2 = 3*pX^3 */
    if (mp_sub(&tmp, &tmp2, &tmp) != MP_OKAY)               { goto error; } /* tmp = tmp - tmp2 */
    if (mp_add(&tmp, &p, &tmp) != MP_OKAY)                  { goto error; } /* tmp = tmp + p */
-   if (mp_read_radix(&p, (unsigned char *)sets[idx].prime, 64) != MP_OKAY)  { goto error; } /* p = prime */
+   if (mp_read_radix(&p, (char *)sets[idx].prime, 64) != MP_OKAY)  { goto error; } /* p = prime */
    if (mp_mod(&tmp, &p, &tmp) != MP_OKAY)                  { goto error; } /* tmp = tmp mod p */
 
    /* now find square root */
@@ -679,12 +686,12 @@ static int compress_y_point(ecc_point *pt, int idx, int *result)
    if (mp_exptmod(&tmp, &tmp2, &p, &tmp) != MP_OKAY)       { goto error; } /* tmp  = (x^3 - 3x + b)^((p+1)/4) mod p */
 
    /* if tmp equals the y point give a 0, otherwise 1 */
-   if (mp_cmp(&tmp, &pt->y) == 0) { 
+   if (mp_cmp(&tmp, &pt->y) == 0) {
       *result = 0;
    } else {
       *result = 1;
    }
-   
+
    res = CRYPT_OK;
    goto done;
 error:
@@ -700,18 +707,18 @@ static int expand_y_point(ecc_point *pt, int idx, int result)
    int res;
 
    _ARGCHK(pt != NULL);
- 
+
    if (mp_init_multi(&tmp, &tmp2, &p, NULL) != MP_OKAY) {
       return CRYPT_MEM;
    }
 
    /* get x^3 - 3x + b */
-   if (mp_read_radix(&p, (unsigned char *)sets[idx].B, 64) != MP_OKAY) { goto error; } /* p = B */
+   if (mp_read_radix(&p, (char *)sets[idx].B, 64) != MP_OKAY) { goto error; } /* p = B */
    if (mp_expt_d(&pt->x, 3, &tmp) != MP_OKAY)              { goto error; } /* tmp = pX^3 */
    if (mp_mul_d(&pt->x, 3, &tmp2) != MP_OKAY)              { goto error; } /* tmp2 = 3*pX^3 */
    if (mp_sub(&tmp, &tmp2, &tmp) != MP_OKAY)               { goto error; } /* tmp = tmp - tmp2 */
    if (mp_add(&tmp, &p, &tmp) != MP_OKAY)                  { goto error; } /* tmp = tmp + p */
-   if (mp_read_radix(&p, (unsigned char *)sets[idx].prime, 64) != MP_OKAY)  { goto error; } /* p = prime */
+   if (mp_read_radix(&p, (char *)sets[idx].prime, 64) != MP_OKAY)  { goto error; } /* p = prime */
    if (mp_mod(&tmp, &p, &tmp) != MP_OKAY)                  { goto error; } /* tmp = tmp mod p */
 
    /* now find square root */
@@ -726,7 +733,7 @@ static int expand_y_point(ecc_point *pt, int idx, int result)
    } else {
       if (mp_sub(&p, &tmp, &pt->y) != MP_OKAY) { goto error; }
    }
-   
+
    res = CRYPT_OK;
    goto done;
 error:
@@ -785,8 +792,8 @@ int ecc_export(unsigned char *out, unsigned long *outlen, int type, ecc_key *key
    _ARGCHK(key != NULL);
 
    /* type valid? */
-   if (key->type != PK_PRIVATE && type == PK_PRIVATE) { 
-      return CRYPT_PK_TYPE_MISMATCH; 
+   if (key->type != PK_PRIVATE && type == PK_PRIVATE) {
+      return CRYPT_PK_TYPE_MISMATCH;
    }
 
    /* output type and magic byte */
@@ -808,7 +815,7 @@ int ecc_export(unsigned char *out, unsigned long *outlen, int type, ecc_key *key
    }
 
    /* check size */
-   if (*outlen < y) { 
+   if (*outlen < y) {
       return CRYPT_BUFFER_OVERFLOW;
    }
 
@@ -832,15 +839,16 @@ int ecc_import(const unsigned char *in, unsigned long inlen, ecc_key *key)
    _ARGCHK(in != NULL);
    _ARGCHK(key != NULL);
 
-   /* check type */
-   if ((err = packet_valid_header((unsigned char *)in, PACKET_SECT_ECC, PACKET_SUB_KEY)) != CRYPT_OK) { 
-      return err;
-   }
-   
+   /* check length */
    if (2+PACKET_SIZE > inlen) {
       return CRYPT_INVALID_PACKET;
    }
 
+   /* check type */
+   if ((err = packet_valid_header((unsigned char *)in, PACKET_SECT_ECC, PACKET_SUB_KEY)) != CRYPT_OK) {
+      return err;
+   }
+
    /* init key */
    if (mp_init_multi(&key->pubkey.x, &key->pubkey.y, &key->k, NULL) != MP_OKAY) {
       return CRYPT_MEM;
@@ -849,9 +857,9 @@ int ecc_import(const unsigned char *in, unsigned long inlen, ecc_key *key)
    y = PACKET_SIZE;
    key->type = (int)in[y++];
    s = (unsigned long)in[y++];
-   
+
    for (x = 0; (s > (unsigned long)sets[x].size) && (sets[x].size != 0); x++);
-   if (sets[x].size == 0) { 
+   if (sets[x].size == 0) {
       err = CRYPT_INVALID_KEYSIZE;
       goto error;
    }
@@ -871,36 +879,36 @@ int ecc_import(const unsigned char *in, unsigned long inlen, ecc_key *key)
 
    /* load x coordinate */
    INPUT_BIGNUM(&key->pubkey.x, in, x, y);
-  
+
    /* load y */
    x = (unsigned long)in[y++];
-   if ((err = expand_y_point(&key->pubkey, key->idx, (int)x)) != CRYPT_OK) { 
-       goto error; 
+   if ((err = expand_y_point(&key->pubkey, key->idx, (int)x)) != CRYPT_OK) {
+       goto error;
    }
 
    if (key->type == PK_PRIVATE) {
       /* load private key */
       INPUT_BIGNUM(&key->k, in, x, y);
    }
-   
+
    /* eliminate private key if public */
    if (key->type == PK_PUBLIC) {
       mp_clear(&key->k);
-   }      
-   
+   }
+
    return CRYPT_OK;
 error:
    mp_clear_multi(&key->pubkey.x, &key->pubkey.y, &key->k, NULL);
    return err;
 }
 
-int ecc_shared_secret(ecc_key *private_key, ecc_key *public_key, 
+int ecc_shared_secret(ecc_key *private_key, ecc_key *public_key,
                       unsigned char *out, unsigned long *outlen)
 {
    unsigned long x, y;
    ecc_point *result;
    mp_int prime;
-   int res, err;
+   int res;
 
    _ARGCHK(private_key != NULL);
    _ARGCHK(public_key != NULL);
@@ -918,17 +926,17 @@ int ecc_shared_secret(ecc_key *private_key, ecc_key *public_key,
 
    /* make new point */
    result = new_point();
-   if (result == NULL) { 
+   if (result == NULL) {
       return CRYPT_MEM;
    }
 
-   if (mp_init(&prime) != MP_OKAY) { 
+   if (mp_init(&prime) != MP_OKAY) {
       del_point(result);
       return CRYPT_MEM;
    }
 
-   if (mp_read_radix(&prime, (unsigned char *)sets[private_key->idx].prime, 64) != MP_OKAY) { goto error; }
-   if ((err = ecc_mulmod(&private_key->k, &public_key->pubkey, result, &prime)) != CRYPT_OK) { res = err; goto done1; }
+   if (mp_read_radix(&prime, (char *)sets[private_key->idx].prime, 64) != MP_OKAY)  { goto error; }
+   if ((res = ecc_mulmod(&private_key->k, &public_key->pubkey, result, &prime)) != CRYPT_OK) { goto done1; }
 
    x = (unsigned long)mp_unsigned_bin_size(&result->x);
    y = (unsigned long)mp_unsigned_bin_size(&result->y);
@@ -938,8 +946,8 @@ int ecc_shared_secret(ecc_key *private_key, ecc_key *public_key,
       goto done1;
    }
    *outlen = x+y;
-   (void)mp_to_unsigned_bin(&result->x, out);
-   (void)mp_to_unsigned_bin(&result->y, out+x);
+   if (mp_to_unsigned_bin(&result->x, out) != MP_OKAY)                                       { goto error; }
+   if (mp_to_unsigned_bin(&result->y, out+x) != MP_OKAY)                                     { goto error; }
 
    res = CRYPT_OK;
    goto done1;
