@@ -25,12 +25,7 @@
 #ifndef _SESSION_H_
 #define _SESSION_H_
 
-#include <stdio.h>
-#ifndef DISABLE_ZLIB
-#include <zlib.h>
-#endif
-
-#include "options.h"
+#include "includes.h"
 #include "buffer.h"
 #include "signkey.h"
 #include "kex.h"
@@ -45,6 +40,7 @@ extern int exitflag;
 void session_cleanup();
 void child_session(int sock, runopts *opts, int childpipe,
 		struct sockaddr *remoteaddr);
+void session_remoteclosed();
 
 struct key_context {
 
@@ -81,7 +77,7 @@ struct sshsession {
 	unsigned char *addrstring; /* the text version of remoteaddr */
 	unsigned char *hostname; /* the remote hostname */
 
-	unsigned int maxfd; /* the maximum file descriptor to check with select() */
+	int maxfd; /* the maximum file descriptor to check with select() */
 	unsigned char *remoteident;
 
 	struct KEXState kexstate;
@@ -124,6 +120,8 @@ struct sshsession {
 	/* channels */
 	struct Channel ** channels; /* these pointers may be null */
 	unsigned int chansize; /* the number of Channel*s allocated for channels */
+	unsigned int chancount; /* the number of active channels, used to determine
+							   if we can cleanup/die */
 
 	struct ChildPid * childpids; /* array of mappings childpid<->channel */
 	unsigned int childpidsize;
