@@ -27,7 +27,7 @@
 
 const struct TermCode termcodes[] = {
 
-		{0}, /* TTY_OP_END */
+		{0, 0}, /* TTY_OP_END */
 		{VINTR, TERMCODE_CONTROLCHAR}, /* control character codes */
 		{VQUIT, TERMCODE_CONTROLCHAR},
 		{VERASE, TERMCODE_CONTROLCHAR},
@@ -41,38 +41,46 @@ const struct TermCode termcodes[] = {
 #ifdef VDSUSP
 		{VDSUSP, TERMCODE_CONTROLCHAR},
 #else
-		{0},
+		{0, 0},
 #endif
 		{VREPRINT, TERMCODE_CONTROLCHAR},
+#ifdef AIX
+		{CERASE, TERMCODE_CONTROLCHAR},
+#else
 		{VWERASE, TERMCODE_CONTROLCHAR},
+#endif
 		{VLNEXT, TERMCODE_CONTROLCHAR},
 #ifdef VFLUSH
 		{VFLUSH, TERMCODE_CONTROLCHAR},
 #else	
-		{0},
+		{0, 0},
 #endif
 #ifdef VSWTCH
 		{VSWTCH, TERMCODE_CONTROLCHAR},
 #else	
-		{0},
+		{0, 0},
 #endif
 #ifdef VSTATUS
 		{VSTATUS, TERMCODE_CONTROLCHAR},
 #else
-		{0},
+		{0, 0},
 #endif
+#ifdef AIX
+		{CKILL, TERMCODE_CONTROLCHAR},
+#else
 		{VDISCARD, TERMCODE_CONTROLCHAR},
-		{0}, /* 19 */
-		{0},
-		{0},
-		{0},
-		{0},
-		{0},
-		{0},
-		{0},
-		{0},
-		{0},
-		{0}, /* 29 */
+#endif
+		{0, 0}, /* 19 */
+		{0, 0},
+		{0, 0},
+		{0, 0},
+		{0, 0},
+		{0, 0},
+		{0, 0},
+		{0, 0},
+		{0, 0},
+		{0, 0},
+		{0, 0}, /* 29 */
 		{IGNPAR, TERMCODE_INPUT}, /* input flags */
 		{PARMRK, TERMCODE_INPUT},
 		{INPCK, TERMCODE_INPUT},
@@ -83,7 +91,7 @@ const struct TermCode termcodes[] = {
 #ifdef IUCLC
 		{IUCLC, TERMCODE_INPUT},
 #else
-		{0},
+		{0, 0},
 #endif
 		{IXON, TERMCODE_INPUT},
 		{IXANY, TERMCODE_INPUT},
@@ -91,22 +99,22 @@ const struct TermCode termcodes[] = {
 #ifdef IMAXBELL
 		{IMAXBELL, TERMCODE_INPUT},
 #else
-		{0},
+		{0, 0},
 #endif
-		{0}, /* 42 */
-		{0},
-		{0},
-		{0},
-		{0},
-		{0},
-		{0},
-		{0}, /* 49 */
+		{0, 0}, /* 42 */
+		{0, 0},
+		{0, 0},
+		{0, 0},
+		{0, 0},
+		{0, 0},
+		{0, 0},
+		{0, 0}, /* 49 */
 		{ISIG, TERMCODE_LOCAL}, /* local flags */
 		{ICANON, TERMCODE_LOCAL},
 #ifdef XCASE
 		{XCASE, TERMCODE_LOCAL},
 #else
-		{0},
+		{0, 0},
 #endif
 		{ECHO, TERMCODE_LOCAL},
 		{ECHOE, TERMCODE_LOCAL},
@@ -118,49 +126,49 @@ const struct TermCode termcodes[] = {
 		{ECHOCTL, TERMCODE_LOCAL},
 		{ECHOKE, TERMCODE_LOCAL},
 		{PENDIN, TERMCODE_LOCAL},
-		{0}, /* 63 */
-		{0},
-		{0},
-		{0},
-		{0},
-		{0},
-		{0}, /* 69 */
+		{0, 0}, /* 63 */
+		{0, 0},
+		{0, 0},
+		{0, 0},
+		{0, 0},
+		{0, 0},
+		{0, 0}, /* 69 */
 		{OPOST, TERMCODE_OUTPUT}, /* output flags */
 #ifdef OLCUC
 		{OLCUC, TERMCODE_OUTPUT},
 #else
-		{0},
+		{0, 0},
 #endif
 		{ONLCR, TERMCODE_OUTPUT},
 #ifdef OCRNL
 		{OCRNL, TERMCODE_OUTPUT},
 #else
-		{0},
+		{0, 0},
 #endif
 #ifdef ONOCR
 		{ONOCR, TERMCODE_OUTPUT},
 #else
-		{0},
+		{0, 0},
 #endif
 #ifdef ONLRET
 		{ONLRET, TERMCODE_OUTPUT},
 #else
-		{0},
+		{0, 0},
 #endif
-		{0}, /* 76 */
-		{0},
-		{0},
-		{0},
-		{0},
-		{0},
-		{0},
-		{0},
-		{0},
-		{0},
-		{0},
-		{0},
-		{0},
-		{0}, /* 89 */
+		{0, 0}, /* 76 */
+		{0, 0},
+		{0, 0},
+		{0, 0},
+		{0, 0},
+		{0, 0},
+		{0, 0},
+		{0, 0},
+		{0, 0},
+		{0, 0},
+		{0, 0},
+		{0, 0},
+		{0, 0},
+		{0, 0}, /* 89 */
 		{CS7, TERMCODE_CONTROL},
 		{CS8, TERMCODE_CONTROL},
 		{PARENB, TERMCODE_CONTROL},
